import { filter, Observable, Observer, Subject } from "rxjs"
import config from '../config/config.json';
import { v4 as uuidv4 } from 'uuid'
import ConsoleLogger from "../utils/log.utils"
import { AdapterInterface, ClientObject, GeneralEvent, ReceiverAdapterInterface, TransmitterAdapterInterface, Transport, TransportService, TransportSet } from "../interface/interface"
import { TransmitterAdapter } from "./adapter.transmitter";
import { ReceiverAdapter } from "./adapter.receiver";
import { AdapterManagerBase } from "../base/adapter.manager.base";

/* Note: There will be a need to use the logic in place for socket utility. Especially for client identification
Will think about that later. Because that mechanism needs to be made universal somehow. If let's say utilizing already
existing transport, there are no logic in place to exchange information to identify connected clients. */

export class AdapterManager extends AdapterManagerBase {
    private console: ConsoleLogger = new ConsoleLogger(`AdapterManager`, ['managers'])

    constructor(event: Subject<GeneralEvent<any>>, browserEnv?: boolean) {
        super()
        this.console.log({ message: `Contructing self...` })
        this.event = event

    }

    public subscribeForAdapters(): Observable<AdapterInterface> {
        return new Observable((observer: Observer<AdapterInterface>) => {
            this.event.pipe(
                // usually a new client is notified from the transport service.
                filter(event => event.event === `New Client`)
            ).subscribe({
                next: (event: GeneralEvent<any>) => {
                    this.handleTransportEvent(event, this.event, event.transport)                                              
                }
            })
        })
    }

    private handleTransportEvent(event: GeneralEvent<ClientObject>, adapterEvent: Subject<GeneralEvent<any>>, transport: Transport): void {
        let transportService: TransportService | undefined = this.transportServiceArray.find(obj => obj.getInfo() === transport)
        if (transportService) {
            let transmitterAdapter: TransmitterAdapterInterface = new TransmitterAdapter(event.data.clientId, event.transport, transportService)
            let receiverAdapter: ReceiverAdapterInterface = new ReceiverAdapter(event.data.clientId, event.transport, transportService)
            adapterEvent.next({
                id: uuidv4(),
                event: 'New Adapter',
                type: 'Adapter Event',
                date: new Date(),
                data: [transmitterAdapter, receiverAdapter]
            } as GeneralEvent<AdapterInterface[]>)
        } else {
            this.console.error({ message: `No ${transport} service is not properly instantiated....` })
            throw new Error(`No ${transport} service is not properly instantiated....`)
        }
    }

}


